//***********************************************************************************
//*        File Name:   seven_seg.v
//*          Version:   1.0
//*             Date:   July 21, 2003
//*   File Hierarchy:   Low level module
//*     Dependencies:   None
//*  
//*         Designer:   Bryan Fletcher
//*          Company:   Memec
//*
//*
//*      Description:   This module does the following:
//*                      1. Divides 25 MHz input clock by 16 using DLL
//*                      2. Creates a 1-second enable signal with a logic counter
//*                      3. Counts 0-9 on 1st 7-segment display
//*                      4. Displays 4-bit binary DIP switch value in hex on
//*                         2nd 7-segment display 
//*                      5. Lights 2 user LEDs based on the 2 push buttons
//*                      6. PUSH[1] also serves as the system reset
//*                         
//************************************************************************************


module seven_seg (CLK_SOCKET, PUSH, DIP, DISPLAY_1, DISPLAY_2, LED);

// Input Declarations
input           CLK_SOCKET;
input    [2:1]  PUSH;
input    [3:0]  DIP;  //DIP[3] is SW3[1] on the board

// Output Declarations
output   [6:0]  DISPLAY_1;
output   [6:0]  DISPLAY_2;
output   [2:1]  LED;

// Input Registers
reg      [3:0]  DIP_d [3:0];
reg      [3:0]  DIP_r;
reg      [3:0]  PUSH1_r;
reg      [3:0]  PUSH2_r;

// Output Registers
reg      [2:1]  LED;
reg      [6:0]  DISPLAY_1;
reg      [6:0]  DISPLAY_2;

// Other Registers
reg      [20:0] led_count;
reg      [3:0]  led_seg_count;
reg             reset;

// Internal signals
wire            led_count_en;
wire            clk_i, clk_fb, clk0, clkdv, clk_div16;

// Register and debounce push buttons and switches
always @(posedge clk_div16)
   begin
     PUSH1_r[0] <= PUSH[1];
     PUSH1_r[1] <= PUSH1_r[0];
     PUSH1_r[2] <= PUSH1_r[1];
     PUSH1_r[3] <= PUSH1_r[2];
     reset      <= &PUSH1_r;
     LED[1]     <= reset;      // Light LED1 based on debounced PUSH1/reset

     PUSH2_r[0] <= PUSH[2];
     PUSH2_r[1] <= PUSH2_r[0];
     PUSH2_r[2] <= PUSH2_r[1];
     PUSH2_r[3] <= PUSH2_r[2];
     LED[2]     <= &PUSH2_r;    // Light LED2 based on debounced PUSH2

     DIP_d[0]      <= DIP;
     DIP_d[1]      <= DIP_d[0];
     DIP_d[2]      <= DIP_d[1];
     DIP_d[3]      <= DIP_d[2];
     DIP_r[3]      <= DIP_d[0][3] & DIP_d[1][3] & DIP_d[2][3] & DIP_d[3][3]; 
     DIP_r[2]      <= DIP_d[0][2] & DIP_d[1][2] & DIP_d[2][2] & DIP_d[3][2]; 
     DIP_r[1]      <= DIP_d[0][1] & DIP_d[1][1] & DIP_d[2][1] & DIP_d[3][1]; 
     DIP_r[0]      <= DIP_d[0][0] & DIP_d[1][0] & DIP_d[2][0] & DIP_d[3][0]; 
   end


/* Using CLKDLL to divide the input 25 MHz clock by 16
   resulting in a 1.5625 MHz internal clock */
IBUFG     U1 ( .I(CLK_SOCKET), .O(clk_i));
 
CLKDLL    dll_div_by_16 ( .CLKIN(clk_i), .CLKFB(clk_fb), .RST(1'b0),
               .CLK0(clk0), .CLK90(), .CLK180(), .CLK270(),
               .CLK2X(), .CLKDV(clkdv), .LOCKED());
 
BUFG      U3 ( .I(clk0), .O(clk_fb));
BUFG      U4 ( .I(clkdv), .O(clk_div16));


//    Segment diagram for the 7-segment LED display (low-enabled).
//
//          0
//       -------
//      5|     |1
//       |     |
//       ---6---
//      4|     |2
//       |     |
//       -------
//          3


// Register outputs
always @(posedge clk_div16 or negedge reset)
   begin
     if(!reset)
       begin
          DISPLAY_1 <= 7'h7f;    //turn off displays during reset
          DISPLAY_2 <= 7'h7f;
       end
     else
       begin
          //   The following block encodes a binary number (0-9) output from
          //   the counter and displays it on 7-segment LED Display #1
          case (led_seg_count)
                  0:   DISPLAY_2   <= ~(7'b0111111);
                  1:   DISPLAY_2   <= ~(7'b0000110);   
                  2:   DISPLAY_2   <= ~(7'b1011011);   
                  3:   DISPLAY_2   <= ~(7'b1001111);   
                  4:   DISPLAY_2   <= ~(7'b1100110);   
                  5:   DISPLAY_2   <= ~(7'b1101101);   
                  6:   DISPLAY_2   <= ~(7'b1111101);   
                  7:   DISPLAY_2   <= ~(7'b0000111);   
                  8:   DISPLAY_2   <= ~(7'b1111111);   
                  9:   DISPLAY_2   <= ~(7'b1101111);   
            default:   DISPLAY_2   <= 7'b1111111;
          endcase


          //   The following block reads the DIP switch and outputs the hex
          //   value on 7-segment LED Display #2
          case (~DIP_r)
                  0:   DISPLAY_1   <= ~(7'b0111111); 
                  1:   DISPLAY_1   <= ~(7'b0000110); 
                  2:   DISPLAY_1   <= ~(7'b1011011); 
                  3:   DISPLAY_1   <= ~(7'b1001111); 
                  4:   DISPLAY_1   <= ~(7'b1100110); 
                  5:   DISPLAY_1   <= ~(7'b1101101); 
                  6:   DISPLAY_1   <= ~(7'b1111101); 
                  7:   DISPLAY_1   <= ~(7'b0000111); 
                  8:   DISPLAY_1   <= ~(7'b1111111); 
                  9:   DISPLAY_1   <= ~(7'b1101111); 
               4'hA:   DISPLAY_1   <= ~(7'b1110111); 
               4'hb:   DISPLAY_1   <= ~(7'b1111100); 
               4'hC:   DISPLAY_1   <= ~(7'b0111001); 
               4'hd:   DISPLAY_1   <= ~(7'b1011110); 
               4'hE:   DISPLAY_1   <= ~(7'b1111001); 
               4'hF:   DISPLAY_1   <= ~(7'b1110001); 
            default:   DISPLAY_1   <= 7'b1111111;
          endcase
       end
   end



//   The following block generates numbers 0-9 to be displayed on the 7-segment
//   display. This 4-bit counter is loaded with the binary value of the on-board
//   DIP switch on reset. Once the reset goes inactive, the count will begin. The
//   counter will hold its last count when the count_hold_n signal goes active. This
//   4-bit counter will only count 0-9 and is reset to zero when the counter reaches
//   decimal 9.
always @(posedge clk_div16 or negedge reset)
   begin
      if (!reset)
            led_seg_count <= 4'hf;
      else if ((led_seg_count == 4'h9) & (led_count_en == 1'b1))
            led_seg_count <= 0;
      else if (led_count_en == 1'b1)
            led_seg_count <= led_seg_count + 1;
   end


//   The following counter is used as a prescaler to the above 4-bit counter.
//   The input clock is divided to enable the 4-bit counter once per second.
always @(posedge clk_div16 or negedge reset)
   begin
      if (!reset)
         led_count <= 0;
      else
         if (led_count_en)
          led_count <= 0;
         else
          led_count <= led_count + 1;
   end

assign   led_count_en   = (led_count == 21'd1_562_500);   // Create 1-second count

endmodule







